/*
 * Tlog-play command-line parsing.
 *
 * vim:nomodifiable
 * 
 * ************************* WARNING! DO NOT EDIT! *************************
 * This file is automatically generated from play_conf_cmd.c.m4.
 * *************************************************************************
 *
 * Copyright (C) 2016 Red Hat
 *
 * This file is part of tlog.
 *
 * Tlog is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * Tlog is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with tlog; if not, write to the Free Software
 * Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA
 */

#include <tlog/play_conf_validate.h>
#include <tlog/play_conf_cmd.h>
#include <tlog/json_misc.h>
#include <tlog/rc.h>
#include <tlog/misc.h>
#include <libgen.h>
#include <stdio.h>
#include <unistd.h>
#include <getopt.h>
#include <string.h>
#include <inttypes.h>
#include <assert.h>

tlog_grc
tlog_play_conf_cmd_help(FILE *stream, const char *progname)
{
    const char *fmt =
       "Usage: %s [OPTION...]\n"
       "Play back a terminal I/O recording done by tlog-rec.\n"
       "\n"
       "General options:\n"
       "    -h, --help                  Output a command-line usage message and exit\n"
       "    -f, --follow                Wait for and play back new messages\n"
       "    -r, --reader=STRING         Use STRING log reader (file/es, default file)\n"
       "\n"
       "File reader options:\n"
       "    --file-path=FILE            Read log from FILE file\n"
       "\n"
       "ElasticSearch reader options:\n"
       "    --es-baseurl=STRING         ElasticSearch URL without query or fragment parts\n"
       "    --es-query=STRING           ElasticSearch query\n"
       "\n";
    if (fprintf(stream, fmt, progname, progname) < 0) {
        return TLOG_GRC_ERRNO;
    } else {
        return TLOG_RC_OK;
    }
}

static tlog_grc
tlog_play_conf_cmd_load_args(const char *progname,
                            struct json_object *conf,
                            int argc, char **argv)
{
    /* Option codes */
    enum opt {
        OPT_HELP = 'h',
        OPT_FOLLOW = 'f',
        OPT_READER = 'r',
        OPT_FILE_PATH = 0x100,
        OPT_ES_BASEURL,
        OPT_ES_QUERY,
    };

    /* Description of short options */
    static const char *shortopts = "h::f::r:";

    /* Description of long options */
    static const struct option longopts[] = {
        {
            .name = "help",
            .val = OPT_HELP,
            .has_arg = optional_argument,
        },
        {
            .name = "follow",
            .val = OPT_FOLLOW,
            .has_arg = optional_argument,
        },
        {
            .name = "reader",
            .val = OPT_READER,
            .has_arg = required_argument,
        },
        {
            .name = "file-path",
            .val = OPT_FILE_PATH,
            .has_arg = required_argument,
        },
        {
            .name = "es-baseurl",
            .val = OPT_ES_BASEURL,
            .has_arg = required_argument,
        },
        {
            .name = "es-query",
            .val = OPT_ES_QUERY,
            .has_arg = required_argument,
        },
        {
            .name = NULL
        }
    };

    tlog_grc grc;
    int optcode;
    const char *optname;
    const char *optpath;
    int64_t val_int;
    struct json_object *val = NULL;
    struct json_object *args = NULL;
    int end;
    int i;

    /* May be unused if there are no INT parameters */
    (void)val_int;
    (void)end;

    /* While there are options */
    while ((optcode = getopt_long(argc, argv, 
                                  shortopts, longopts, NULL)) >= 0) {
        switch (optcode) {
        case OPT_HELP:
            optname = "-h/--help";
            optpath = "help";
            if (optarg == NULL ||
                strcasecmp(optarg, "yes") == 0 ||
                strcasecmp(optarg, "on") == 0 ||
                strcasecmp(optarg, "true") == 0) {
                val = json_object_new_boolean(true);
            } else if (strcasecmp(optarg, "no") == 0 ||
                       strcasecmp(optarg, "off") == 0 ||
                       strcasecmp(optarg, "false") == 0) {
                val = json_object_new_boolean(false);
            } else {
                fprintf(stderr,
                        "Invalid %s option value: %s,\n"
                        "expecting yes/on/true or no/off/false.\n",
                        optname, optarg);
                tlog_play_conf_cmd_help(stderr, progname);
                grc = TLOG_RC_FAILURE;
                goto cleanup;
            }
            break;

        case OPT_FOLLOW:
            optname = "-f/--follow";
            optpath = "follow";
            if (optarg == NULL ||
                strcasecmp(optarg, "yes") == 0 ||
                strcasecmp(optarg, "on") == 0 ||
                strcasecmp(optarg, "true") == 0) {
                val = json_object_new_boolean(true);
            } else if (strcasecmp(optarg, "no") == 0 ||
                       strcasecmp(optarg, "off") == 0 ||
                       strcasecmp(optarg, "false") == 0) {
                val = json_object_new_boolean(false);
            } else {
                fprintf(stderr,
                        "Invalid %s option value: %s,\n"
                        "expecting yes/on/true or no/off/false.\n",
                        optname, optarg);
                tlog_play_conf_cmd_help(stderr, progname);
                grc = TLOG_RC_FAILURE;
                goto cleanup;
            }
            break;

        case OPT_READER:
            optname = "-r/--reader";
            optpath = "reader";
            {
                const char *list[] = {"file",
                                      "es"};
                size_t i;
                assert(optarg != NULL);
                if (optarg == NULL) {
                    fprintf(stderr, "Option %s has no value.\n", optname);
                    grc = TLOG_RC_FAILURE;
                    goto cleanup;
                }
                for (i = 0;
                     i < TLOG_ARRAY_SIZE(list) && strcmp(optarg, list[i]) != 0;
                     i++);
                if (i >= TLOG_ARRAY_SIZE(list)) {
                    fprintf(stderr, "Invalid %s option value: %s\n",
                            optname, optarg);
                    grc = TLOG_RC_FAILURE;
                    tlog_play_conf_cmd_help(stderr, progname);
                    goto cleanup;
                }
            }
            val = json_object_new_string(optarg);
            break;

        case OPT_FILE_PATH:
            optname = "--file-path";
            optpath = "file.path";
            assert(optarg != NULL);
            if (optarg == NULL) {
                fprintf(stderr, "Option %s has no value.\n", optname);
                grc = TLOG_RC_FAILURE;
                goto cleanup;
            }
            val = json_object_new_string(optarg);
            break;

        case OPT_ES_BASEURL:
            optname = "--es-baseurl";
            optpath = "es.baseurl";
            assert(optarg != NULL);
            if (optarg == NULL) {
                fprintf(stderr, "Option %s has no value.\n", optname);
                grc = TLOG_RC_FAILURE;
                goto cleanup;
            }
            val = json_object_new_string(optarg);
            break;

        case OPT_ES_QUERY:
            optname = "--es-query";
            optpath = "es.query";
            assert(optarg != NULL);
            if (optarg == NULL) {
                fprintf(stderr, "Option %s has no value.\n", optname);
                grc = TLOG_RC_FAILURE;
                goto cleanup;
            }
            val = json_object_new_string(optarg);
            break;

        case '?':
            grc = TLOG_RC_FAILURE;
            tlog_play_conf_cmd_help(stderr, progname);
            goto cleanup;

        default:
            fprintf(stderr, "Unknown option code: %d\n", optcode);
            grc = TLOG_RC_FAILURE;
            goto cleanup;
        }

        if (val == NULL) {
            grc = TLOG_GRC_ERRNO;
            fprintf(stderr, "Failed creating %s option value: %s\n",
                    optname, tlog_grc_strerror(grc));
            goto cleanup;
        }
        grc = tlog_json_object_object_add_path(conf, optpath, val);
        if (grc != TLOG_RC_OK) {
            fprintf(stderr, "Failed storing %s option value: %s\n",
                    optname, tlog_grc_strerror(grc));
            goto cleanup;
        }
        val = NULL;
    }

    /* Add other arguments */
    args = json_object_new_array();
    if (args == NULL) {
        grc = TLOG_GRC_ERRNO;
        fprintf(stderr, "Failed creating positional argument list: %s\n",
                tlog_grc_strerror(grc));
        goto cleanup;
    }
    for (i = 0; optind < argc; i++, optind++) {
        val = json_object_new_string(argv[optind]);
        if (val == NULL) {
            grc = TLOG_GRC_ERRNO;
            fprintf(stderr, "Failed creating argument value: %s\n",
                    tlog_grc_strerror(grc));
            goto cleanup;
        }
        if (json_object_array_put_idx(args, i, val) < 0) {
            grc = TLOG_GRC_ERRNO;
            fprintf(stderr, "Failed storing argument #%d: %s\n",
                    i, tlog_grc_strerror(grc));
            goto cleanup;
        }
        val = NULL;
    }
    grc = tlog_json_object_object_add_path(conf, "args", args);
    if (grc != TLOG_RC_OK) {
        fprintf(stderr, "Failed storing argument list: %s\n",
                tlog_grc_strerror(grc));
        goto cleanup;
    }
    args = NULL;

    /* Validate the result */
    grc = tlog_play_conf_validate(conf, TLOG_CONF_ORIGIN_ARGS);
    if (grc != TLOG_RC_OK) {
        goto cleanup;
    }

    grc = TLOG_RC_OK;

cleanup:
    json_object_put(args);
    json_object_put(val);
    return grc;
}

tlog_grc
tlog_play_conf_cmd_load(char **pprogname, struct json_object **pconf,
                        int argc, char **argv)
{
    tlog_grc grc;
    char *progpath = NULL;
    char *progname = NULL;
    struct json_object *conf = NULL;

    /* Create empty configuration */
    conf = json_object_new_object();
    if (conf == NULL) {
        grc = TLOG_GRC_ERRNO;
        fprintf(stderr, "Failed creating configuration object: %s\n",
                tlog_grc_strerror(grc));
        goto cleanup;
    }

    /* Extract program name */
    progpath = strdup(argv[0]);
    if (progpath == NULL) {
        grc = TLOG_GRC_ERRNO;
        fprintf(stderr, "Failed allocating a copy of program path: %s\n",
                tlog_grc_strerror(grc));
        goto cleanup;
    }
    progname = strdup(basename(progpath));
    if (progname == NULL) {
        grc = TLOG_GRC_ERRNO;
        fprintf(stderr, "Failed allocating program name: %s\n",
                tlog_grc_strerror(grc));
        goto cleanup;
    }

    /* Extract options and positional arguments */
    grc = tlog_play_conf_cmd_load_args(progname, conf, argc, argv);
    if (grc != TLOG_RC_OK) {
        goto cleanup;
    }

    /* Validate the result */
    grc = tlog_play_conf_validate(conf, TLOG_CONF_ORIGIN_ARGS);
    if (grc != TLOG_RC_OK) {
        goto cleanup;
    }

    *pprogname = progname;
    progname = NULL;
    *pconf = conf;
    conf = NULL;
    grc = TLOG_RC_OK;

cleanup:
    free(progname);
    free(progpath);
    json_object_put(conf);
    return grc;
}
